@@ -5,6 +5,7 @@ use crate::CH_WIDTH;
 use crate::COMPONENT_NAME;
 use css_colors::Color;
 use history::Recorded;
+use nalgebra::Point2;
 use sauron::html::attributes;
 use sauron::html::units;
 use sauron::jss::jss_ns;
@@ -60,15 +61,16 @@ pub struct Editor<XMSG> {
     editor_element: Option<web_sys::Element>,
     composed_key: Option<char>,
     last_char_count: Option<usize>,
-    editor_offset: Option<(f32, f32)>,
+    is_selecting: bool,
+    selection_start: Option<Point2<i32>>,
+    selection_end: Option<Point2<i32>>,
 }
 
 impl<XMSG> Editor<XMSG> {
-    pub fn from_str(content: &str, syntax_token: &str) -> Self {
-        let options = Options::default();
+    pub fn from_str(options: Options, content: &str) -> Self {
         let editor = Editor {
             options: options.clone(),
-            text_buffer: TextBuffer::from_str(options, content, syntax_token),
+            text_buffer: TextBuffer::from_str(options, content),
             page_size: 10,
             recorded: Recorded::new(),
             measurements: None,
@@ -82,7 +84,9 @@ impl<XMSG> Editor<XMSG> {
             editor_element: None,
             composed_key: None,
             last_char_count: None,
-            editor_offset: None,
+            is_selecting: false,
+            selection_start: None,
+            selection_end: None,
         };
         editor
     }
@@ -93,11 +97,7 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
         match msg {
             Msg::EditorMounted(target_node) => {
                 let element: &web_sys::Element = target_node.unchecked_ref();
-                let rect = element.get_bounding_client_rect();
-                let editor_x = rect.x().round() as f32;
-                let editor_y = rect.y().round() as f32;
                 self.editor_element = Some(element.clone());
-                self.editor_offset = Some((editor_x, editor_y));
                 Effects::none()
             }
             Msg::TextareaMounted(target_node) => {
@@ -139,38 +139,76 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                     self.clear_hidden_textarea();
                 }
                 self.last_char_count = Some(char_count);
-                Effects::none().measure()
+
+                let extern_msgs = self.emit_on_change_listeners();
+                Effects::with_external(extern_msgs).measure()
             }
             Msg::TextareaKeydown(ke) => {
+                let is_ctrl = ke.ctrl_key();
+                log::trace!("text area key down... {}", is_ctrl);
                 // don't process key presses when
                 // CTRL key is pressed.
-                if !ke.ctrl_key() {
-                    let key = ke.key();
-                    self.process_keypresses(&ke);
-                    if key.chars().count() == 1 {
-                        let c = key.chars().next().expect("must be only 1 chr");
-                        self.command_insert_char(c);
-                        self.clear_hidden_textarea();
-                        let extern_msgs = self.emit_on_change_listeners();
-                        return Effects::with_external(extern_msgs).measure();
+                let key = ke.key();
+                self.process_keypresses(&ke);
+                if key.chars().count() == 1 {
+                    let c = key.chars().next().expect("must be only 1 chr");
+                    match c {
+                        'c' if is_ctrl => {
+                            //self.copy_to_clipboard();
+                            self.textarea_exec_copy();
+                            self.clear_hidden_textarea();
+                        }
+                        'v' if is_ctrl => {
+                            log::trace!("pasting is handled");
+                            self.clear_hidden_textarea();
+                        }
+                        _ => {
+                            self.command_insert_char(c);
+                            self.clear_hidden_textarea();
+                        }
+                    }
+                }
+                let extern_msgs = self.emit_on_change_listeners();
+                Effects::with_external(extern_msgs).measure()
+            }
+            Msg::Mouseup(client_x, client_y) => {
+                let cursor = self.client_to_cursor(client_x, client_y);
+                self.selection_end = Some(cursor);
+                if let (Some(start), Some(end)) =
+                    (self.selection_start, self.selection_end)
+                {
+                    self.is_selecting = false;
+                    self.command_set_selection(start, end);
+                }
+                self.command_set_position(cursor.x, cursor.y);
+
+                if let Some(selected_text) = self.text_buffer.selected_text() {
+                    log::trace!("selected text: \n{}", selected_text);
+                }
+                Effects::none().measure()
+            }
+            Msg::Mousedown(client_x, client_y) => {
+                let cursor = self.client_to_cursor(client_x, client_y);
+                self.is_selecting = true;
+                self.selection_start = Some(cursor);
+                self.command_set_position(cursor.x, cursor.y);
+                Effects::none().measure()
+            }
+            Msg::Mousemove(client_x, client_y) => {
+                if self.is_selecting {
+                    let cursor = self.client_to_cursor(client_x, client_y);
+                    self.selection_end = Some(cursor);
+
+                    if let Some(start) = self.selection_start {
+                        self.command_set_selection(start, cursor);
                     }
                 }
                 Effects::none()
             }
-            Msg::Mouseup(client_x, client_y) => {
-                let (x, y) = self.client_to_cursor(client_x, client_y);
-                self.text_buffer.set_position(x, y);
-                Effects::none().measure()
-            }
-            Msg::Mousedown(client_x, client_y) => {
-                let (x, y) = self.client_to_cursor(client_x, client_y);
-                self.text_buffer.set_position(x, y);
-                Effects::none().measure()
-            }
-            Msg::Mousemove(_client_x, _client_y) => Effects::none(),
             Msg::Paste(text_content) => {
                 self.command_insert_text(&text_content);
-                Effects::none()
+                let extern_msgs = self.emit_on_change_listeners();
+                Effects::with_external(extern_msgs)
             }
             Msg::CopiedSelected => Effects::none(),
             Msg::MoveCursor(_line, _col) => Effects::none(),
@@ -190,10 +228,9 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                     let c = key.chars().next().expect("must be only 1 chr");
                     self.text_buffer.command_insert_char(c);
                     self.text_buffer.rehighlight();
-                    let extern_msgs = self.emit_on_change_listeners();
-                    return Effects::with_external(extern_msgs);
                 }
-                Effects::none()
+                let extern_msgs = self.emit_on_change_listeners();
+                Effects::with_external(extern_msgs)
             }
         }
     }
@@ -226,6 +263,14 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                 cursor: "text",
                 width: percent(100),
                 height: percent(100),
+                white_space: "normal",
+            },
+
+            "pre code":{
+                white_space: "pre",
+                word_spacing: "normal",
+                word_break: "normal",
+                word_wrap: "normal",
             },
 
             // paste area hack, we don't want to use
@@ -257,6 +302,7 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                 bottom: 0,
                 display: "flex",
                 flex_direction: "flex-end",
+                user_select: "none",
             },
 
         };
@@ -277,6 +323,11 @@ impl<XMSG> Editor<XMSG> {
         self.text_buffer.rehighlight();
     }
 
+    pub fn command_replace_char(&mut self, ch: char) {
+        self.text_buffer.command_replace_char(ch);
+        self.text_buffer.rehighlight();
+    }
+
     fn command_break_line(&mut self) {
         self.text_buffer.command_break_line();
         self.text_buffer.rehighlight();
@@ -287,6 +338,53 @@ impl<XMSG> Editor<XMSG> {
         self.text_buffer.rehighlight();
     }
 
+    pub fn command_set_position(&mut self, cursor_x: i32, cursor_y: i32) {
+        if self.text_buffer.in_bounds(Point2::new(cursor_x, cursor_y)) {
+            self.text_buffer
+                .set_position(cursor_x as usize, cursor_y as usize);
+        }
+    }
+
+    fn command_set_selection(&mut self, start: Point2<i32>, end: Point2<i32>) {
+        if self.text_buffer.in_bounds(start) && self.text_buffer.in_bounds(end)
+        {
+            let start = Point2::new(start.x as usize, start.y as usize);
+            let end = Point2::new(end.x as usize, end.y as usize);
+            self.text_buffer.set_selection(start, end);
+        }
+    }
+
+    /// this is for newer browsers
+    /// This doesn't work on webkit2
+    #[allow(unused)]
+    #[cfg(web_sys_unstable_apis)]
+    #[cfg(feature = "with-navigator-clipboard")]
+    fn copy_to_clipboard(&self) {
+        if let Some(selected_text) = self.text_buffer.selected_text() {
+            let navigator = sauron::window().navigator();
+            if let Some(clipboard) = navigator.clipboard() {
+                let _ = clipboard.write_text(&selected_text);
+            }
+        }
+    }
+    /// execute copy on the selected textarea
+    /// this works even on older browser
+    fn textarea_exec_copy(&self) {
+        use sauron::web_sys::HtmlDocument;
+
+        if let Some(selected_text) = self.text_buffer.selected_text() {
+            if let Some(ref hidden_textarea) = self.hidden_textarea {
+                hidden_textarea.set_value(&selected_text);
+
+                hidden_textarea.select();
+                let html_document: HtmlDocument =
+                    sauron::document().unchecked_into();
+                html_document.exec_command("copy").expect("must copy");
+                hidden_textarea.set_value("");
+            }
+        }
+    }
+
     fn process_keypresses(&mut self, ke: &web_sys::KeyboardEvent) {
         let key = ke.key();
         match &*key {
@@ -372,43 +470,60 @@ impl<XMSG> Editor<XMSG> {
         extern_msgs
     }
 
+    ///TODO: This call the the real DOM, maybe create recalculate this everytime
+    /// a scroll even is made
+    fn editor_offset(&self) -> Option<Point2<f32>> {
+        if let Some(ref editor_element) = self.editor_element {
+            log::trace!("calculating editor offset..");
+            let rect = editor_element.get_bounding_client_rect();
+            let editor_x = rect.x().round() as f32;
+            let editor_y = rect.y().round() as f32;
+            Some(Point2::new(editor_x, editor_y))
+        } else {
+            None
+        }
+    }
+
     /// convert screen coordinate to cursor position
-    fn client_to_cursor(&self, client_x: i32, client_y: i32) -> (usize, usize) {
+    pub fn client_to_cursor(
+        &self,
+        client_x: i32,
+        client_y: i32,
+    ) -> Point2<i32> {
         let numberline_wide = self.text_buffer.get_numberline_wide() as f32;
-        let (editor_x, editor_y) =
-            self.editor_offset.expect("must have editor offset");
-        let col = (client_x as f32 - editor_x + self.window_scroll_left)
+        let editor = self.editor_offset().expect("must have editor offset");
+        let col = (client_x as f32 - editor.x + self.window_scroll_left)
             / CH_WIDTH as f32
             - numberline_wide;
-        let line = (client_y as f32 - editor_y + self.window_scroll_top)
+        let line = (client_y as f32 - editor.y + self.window_scroll_top)
             / CH_HEIGHT as f32;
-        let x = col.floor() as usize;
-        let y = line.floor() as usize;
-        (x, y)
+        let x = col.floor() as i32;
+        let y = line.floor() as i32;
+        Point2::new(x, y)
     }
 
     /// convert current cursor position to client coordinate relative to the editor div
-    fn cursor_to_client(&self) -> (i32, i32) {
+    pub fn cursor_to_client(&self) -> Point2<i32> {
         let numberline_wide = self.text_buffer.get_numberline_wide() as f32;
-        let (x, y) = self.text_buffer.get_position();
+        let cursor = self.text_buffer.get_position();
 
-        let top = y as i32 * CH_HEIGHT as i32;
-        let left = (x as i32 + numberline_wide as i32) * CH_WIDTH as i32;
+        let top = cursor.y as i32 * CH_HEIGHT as i32;
+        let left = (cursor.x as i32 + numberline_wide as i32) * CH_WIDTH as i32;
 
-        (left, top)
+        Point2::new(left, top)
     }
 
     fn view_hidden_textarea(&self) -> Node<Msg> {
         let class_ns = |class_names| {
             attributes::class_namespaced(COMPONENT_NAME, class_names)
         };
-        let (cursor_x, cursor_y) = self.cursor_to_client();
+        let cursor = self.cursor_to_client();
         div(
             [
                 class_ns("hidden_textarea_wrapper"),
                 style! {
-                    top: px(cursor_y),
-                    left: px(cursor_x),
+                    top: px(cursor.y),
+                    left: px(cursor.x),
                     z_index: 99,
                 },
             ],
@@ -459,13 +574,13 @@ impl<XMSG> Editor<XMSG> {
         let class_ns = |class_names| {
             attributes::class_namespaced(COMPONENT_NAME, class_names)
         };
-        let (left, top) = self.cursor_to_client();
+        let cursor = self.cursor_to_client();
         div(
             [
                 class_ns("virtual_cursor"),
                 style! {
-                    top: px(top),
-                    left: px(left),
+                    top: px(cursor.y),
+                    left: px(cursor.x),
                 },
             ],
             [],
@@ -476,7 +591,7 @@ impl<XMSG> Editor<XMSG> {
         let class_ns = |class_names| {
             attributes::class_namespaced(COMPONENT_NAME, class_names)
         };
-        let (x_pos, y_pos) = self.text_buffer.get_position();
+        let cursor = self.text_buffer.get_position();
         div(
             [
                 class_ns("status"),
@@ -496,18 +611,25 @@ impl<XMSG> Editor<XMSG> {
                 },
             ],
             [
-                text!("line: {}, col: {}  |", y_pos + 1, x_pos + 1),
+                text!("line: {}, col: {}  |", cursor.y + 1, cursor.x + 1),
                 if let Some(measurements) = &self.measurements {
                     text!(
                         "patches: {} | nodes: {} | view time: {}ms | patch time: {}ms | update time: {}ms",
                         measurements.total_patches,
                         measurements.view_node_count,
-                        measurements.build_view_took,
-                        measurements.dom_update_took,
+                        measurements.build_view_took.round(),
+                        measurements.dom_update_took.round(),
                         measurements.total_time.round()
                     )
                 } else {
-                    text("")
+                    comment("")
+                },
+                if let (Some(start), Some(end)) =
+                    (self.selection_start, self.selection_end)
+                {
+                    text!("| selection start: {} end: {}", start, end)
+                } else {
+                    text("| no selection")
                 },
             ],
         )
