@@ -8,6 +8,7 @@ use css_colors::rgba;
 use css_colors::Color;
 use css_colors::RGBA;
 use line::Line;
+use nalgebra::Point2;
 use range::Range;
 use sauron::html::attributes;
 use sauron::jss::jss_ns;
@@ -29,16 +30,12 @@ pub struct TextBuffer {
     options: Options,
     lines: Vec<Line>,
     text_highlighter: TextHighlighter,
-    x_pos: usize,
-    y_pos: usize,
+    cursor: Point2<usize>,
     #[allow(unused)]
-    selection_start: Option<(usize, usize)>,
+    selection_start: Option<Point2<usize>>,
     #[allow(unused)]
-    selection_end: Option<(usize, usize)>,
+    selection_end: Option<Point2<usize>>,
     focused_cell: Option<FocusCell>,
-    /// the language to be used for highlighting the content
-    #[allow(unused)]
-    syntax_token: String,
 }
 
 #[derive(Clone, Copy, Debug)]
@@ -50,35 +47,139 @@ struct FocusCell {
 }
 
 impl TextBuffer {
-    pub fn from_str(
-        options: Options,
-        content: &str,
-        syntax_token: &str,
-    ) -> Self {
+    pub fn from_str(options: Options, content: &str) -> Self {
         let mut text_highlighter = TextHighlighter::default();
         if let Some(theme_name) = &options.theme_name {
+            log::trace!("Selecting theme: {}", theme_name);
             text_highlighter.select_theme(theme_name);
         }
         let mut this = Self {
             lines: Self::highlight_content(
                 content,
                 &text_highlighter,
-                syntax_token,
+                &options.syntax_token,
             ),
             text_highlighter,
-            x_pos: 0,
-            y_pos: 0,
+            cursor: Point2::new(0, 0),
             selection_start: None,
             selection_end: None,
             focused_cell: None,
             options,
-            syntax_token: syntax_token.to_string(),
         };
 
         this.calculate_focused_cell();
         this
     }
 
+    pub fn clear(&mut self) {
+        self.lines.clear();
+    }
+
+    pub fn set_selection(&mut self, start: Point2<usize>, end: Point2<usize>) {
+        self.selection_start = Some(start);
+        self.selection_end = Some(end);
+    }
+
+    /// return the min and max selection bound
+    pub fn normalize_selection(
+        &self,
+    ) -> Option<(Point2<usize>, Point2<usize>)> {
+        if let (Some(start), Some(end)) =
+            (self.selection_start, self.selection_end)
+        {
+            Some(util::normalize_points(start, end))
+        } else {
+            None
+        }
+    }
+    /// check if this cell is within the selection of the textarea
+    pub(crate) fn in_selection(
+        &self,
+        line_index: usize,
+        range_index: usize,
+        cell_index: usize,
+    ) -> bool {
+        let x = self.lines[line_index]
+            .calc_range_cell_index_to_x(range_index, cell_index);
+        let y = line_index;
+
+        if let Some((start, end)) = self.normalize_selection() {
+            if self.options.use_block_mode {
+                x >= start.x && x <= end.x && y >= start.y && y <= end.y
+            } else {
+                if y > start.y && y < end.y {
+                    true
+                } else {
+                    let same_start_line = y == start.y;
+                    let same_end_line = y == end.y;
+
+                    if same_start_line && same_end_line {
+                        x >= start.x && x <= end.x
+                    } else if same_start_line {
+                        x >= start.x
+                    } else if same_end_line {
+                        x <= end.x
+                    } else {
+                        false
+                    }
+                }
+            }
+        } else {
+            false
+        }
+    }
+
+    pub(crate) fn selected_text(&self) -> Option<String> {
+        if let (Some(start), Some(_end)) =
+            (self.selection_start, self.selection_end)
+        {
+            let mut buffer = TextBuffer::from_str(Options::default(), "");
+            for (line_index, line) in self.lines.iter().enumerate() {
+                let y = line_index;
+                for (range_index, range) in line.ranges.iter().enumerate() {
+                    for (cell_index, cell) in range.cells.iter().enumerate() {
+                        let x = line.calc_range_cell_index_to_x(
+                            range_index,
+                            cell_index,
+                        );
+                        if self.in_selection(
+                            line_index,
+                            range_index,
+                            cell_index,
+                        ) {
+                            if self.options.use_block_mode {
+                                buffer.insert_char(
+                                    x - start.x,
+                                    y - start.y,
+                                    cell.ch,
+                                );
+                            } else {
+                                buffer.insert_char(x, y - start.y, cell.ch);
+                            }
+                        }
+                    }
+                }
+            }
+            Some(buffer.to_string())
+        } else {
+            None
+        }
+    }
+
+    /// check if x and y is in the bounds of the texteditor longest column
+    pub fn in_bounds(&self, point: Point2<i32>) -> bool {
+        let bound = self.bounds();
+        point.x >= 0 && point.y >= 0 && point.x <= bound.x && point.y <= bound.y
+    }
+
+    /// calculate the bounds of the text_buffer
+    pub fn bounds(&self) -> Point2<i32> {
+        let total_lines = self.lines.len() as i32;
+        let max_column =
+            self.lines.iter().map(|line| line.width).max().unwrap_or(0) as i32;
+        Point2::new(max_column, total_lines)
+    }
+
     pub fn set_options(&mut self, options: Options) {
         self.options = options;
     }
@@ -117,10 +218,10 @@ impl TextBuffer {
     }
 
     fn find_focused_cell(&self) -> Option<FocusCell> {
-        let line_index = self.y_pos;
+        let line_index = self.cursor.y;
         if let Some(line) = self.lines.get(line_index) {
             if let Some((range_index, cell_index)) =
-                line.calc_range_cell_index_position(self.x_pos)
+                line.calc_range_cell_index_position(self.cursor.x)
             {
                 if let Some(range) = line.ranges.get(range_index) {
                     return Some(FocusCell {
@@ -230,8 +331,18 @@ impl TextBuffer {
         let class_number_wide =
             format!("number_wide{}", self.numberline_wide());
 
-        let code_attributes =
-            vec![class_ns("code"), class_ns(&class_number_wide)];
+        let theme_background =
+            self.theme_background().unwrap_or(rgba(0, 0, 255, 1.0));
+
+        let code_attributes = [
+            class_ns("code"),
+            class_ns(&class_number_wide),
+            if self.options.use_background {
+                style! {background: theme_background.to_css()}
+            } else {
+                empty_attr()
+            },
+        ];
 
         let rendered_lines = self
             .lines
@@ -248,8 +359,8 @@ impl TextBuffer {
             // but in firefox, it creates a double line when select-copying the text
             // whe need to use <pre><code> in order for typing whitespace works.
             pre(
-                vec![class_ns("code_wrapper")],
-                vec![code(code_attributes, rendered_lines)],
+                [class_ns("code_wrapper")],
+                [code(code_attributes, rendered_lines)],
             )
         }
     }
@@ -260,8 +371,6 @@ impl TextBuffer {
             .unwrap_or(rgba(100, 100, 100, 0.5));
 
         let cursor_color = self.cursor_color().unwrap_or(rgba(255, 0, 0, 1.0));
-        let theme_background =
-            self.theme_background().unwrap_or(rgba(0, 0, 255, 1.0));
 
         jss_ns! {COMPONENT_NAME,
             ".code_wrapper": {
@@ -270,10 +379,12 @@ impl TextBuffer {
 
             ".code": {
                 position: "relative",
-                background: theme_background.to_css(),
                 font_size: px(14),
                 cursor: "text",
                 display: "block",
+                // to make the background color extend to the longest line, otherwise only the
+                // longest lines has a background-color leaving the shorter lines ugly
+                min_width: "max-content",
             },
 
             ".line_block": {
@@ -421,7 +532,6 @@ impl TextBuffer {
 }
 
 /// text manipulation
-#[cfg(feature = "with-dom")]
 impl TextBuffer {
     /// the total number of lines of this text canvas
     pub(crate) fn total_lines(&self) -> usize {
@@ -439,7 +549,7 @@ impl TextBuffer {
         self.lines = Self::highlight_content(
             &self.to_string(),
             &self.text_highlighter,
-            &self.syntax_token,
+            &self.options.syntax_token,
         );
     }
 
@@ -496,7 +606,7 @@ impl TextBuffer {
     }
 
     /// insert a character at this x and y and move cells after it to the right
-    pub(crate) fn insert_char(&mut self, x: usize, y: usize, ch: char) {
+    pub fn insert_char(&mut self, x: usize, y: usize, ch: char) {
         self.assert_chars(ch);
         self.ensure_cell_exist(x, y);
 
@@ -526,20 +636,18 @@ impl TextBuffer {
             new_col = 0;
             new_line += 1;
             self.break_line(new_col, new_line);
-            self.y_pos = new_line;
+            self.cursor.y = new_line;
         }
     }
 
     /// replace the character at this location
-    #[allow(unused)]
-    pub(crate) fn replace_char(&mut self, x: usize, y: usize, ch: char) {
+    pub fn replace_char(&mut self, x: usize, y: usize, ch: char) {
         self.assert_chars(ch);
-        self.ensure_cell_exist(x, y);
+        self.ensure_cell_exist(x + 1, y);
 
         let (range_index, cell_index) = self.lines[y]
             .calc_range_cell_index_position(x)
             .expect("the range_index and cell_index must have existed at this point");
-
         self.lines[y].replace_char(range_index, cell_index, ch);
     }
 
@@ -578,66 +686,70 @@ impl TextBuffer {
         self.lines.insert(line_index, line);
     }
 
-    pub(crate) fn get_position(&self) -> (usize, usize) {
-        (self.x_pos, self.y_pos)
+    pub(crate) fn get_position(&self) -> Point2<usize> {
+        self.cursor
     }
 }
 
-#[cfg(feature = "with-dom")]
 /// Command implementation here
 impl TextBuffer {
     pub(crate) fn command_insert_char(&mut self, ch: char) {
-        self.insert_char(self.x_pos, self.y_pos, ch);
+        self.insert_char(self.cursor.x, self.cursor.y, ch);
         let width = ch.width().expect("must have a unicode width");
         self.move_x(width);
     }
+
+    pub(crate) fn command_replace_char(&mut self, ch: char) {
+        self.replace_char(self.cursor.x, self.cursor.y, ch);
+    }
+
     pub(crate) fn command_insert_text(&mut self, text: &str) {
         use unicode_width::UnicodeWidthStr;
-        self.insert_text(self.x_pos, self.y_pos, text);
+        self.insert_text(self.cursor.x, self.cursor.y, text);
         self.move_x(text.width());
     }
     pub(crate) fn move_left(&mut self) {
-        self.x_pos = self.x_pos.saturating_sub(1);
+        self.cursor.x = self.cursor.x.saturating_sub(1);
         self.calculate_focused_cell();
     }
     pub(crate) fn move_left_start(&mut self) {
-        self.x_pos = 0;
+        self.cursor.x = 0;
         self.calculate_focused_cell();
     }
     pub(crate) fn move_right(&mut self) {
-        self.x_pos = self.x_pos.saturating_add(1);
+        self.cursor.x = self.cursor.x.saturating_add(1);
         self.calculate_focused_cell();
     }
     pub(crate) fn move_x(&mut self, x: usize) {
-        self.x_pos = self.x_pos.saturating_add(x);
+        self.cursor.x = self.cursor.x.saturating_add(x);
         self.calculate_focused_cell();
     }
     pub(crate) fn move_up(&mut self) {
-        self.y_pos = self.y_pos.saturating_sub(1);
+        self.cursor.y = self.cursor.y.saturating_sub(1);
         self.calculate_focused_cell();
     }
     pub(crate) fn move_down(&mut self) {
-        self.y_pos = self.y_pos.saturating_add(1);
+        self.cursor.y = self.cursor.y.saturating_add(1);
         self.calculate_focused_cell();
     }
     pub(crate) fn set_position(&mut self, x: usize, y: usize) {
-        self.x_pos = x;
-        self.y_pos = y;
+        self.cursor.x = x;
+        self.cursor.y = y;
         self.calculate_focused_cell();
     }
     pub(crate) fn command_break_line(&mut self) {
-        self.break_line(self.x_pos, self.y_pos);
+        self.break_line(self.cursor.x, self.cursor.y);
         self.move_left_start();
         self.move_down();
     }
     pub(crate) fn command_delete_back(&mut self) {
-        if self.x_pos > 0 {
-            self.delete_char(self.x_pos.saturating_sub(1), self.y_pos);
+        if self.cursor.x > 0 {
+            self.delete_char(self.cursor.x.saturating_sub(1), self.cursor.y);
             self.move_left();
         }
     }
     pub(crate) fn command_delete_forward(&mut self) {
-        self.delete_char(self.x_pos, self.y_pos);
+        self.delete_char(self.cursor.x, self.cursor.y);
         self.calculate_focused_cell();
     }
 }
@@ -677,7 +789,7 @@ mod test {
 
     #[test]
     fn test_ensure_line_exist() {
-        let mut buffer = TextBuffer::from_str(Options::default(), "", "txt");
+        let mut buffer = TextBuffer::from_str(Options::default(), "");
         buffer.ensure_line_exist(10);
         assert!(buffer.lines.get(10).is_some());
         assert_eq!(buffer.total_lines(), 11);
