@@ -61,6 +61,22 @@ impl Line {
         None
     }
 
+    /// calculate the x position from range_index and cell_index of this line
+    pub(crate) fn calc_range_cell_index_to_x(
+        &self,
+        range_index: usize,
+        cell_index: usize,
+    ) -> usize {
+        let mut width = 0;
+        for range in self.ranges[0..range_index].iter() {
+            width += range.width;
+        }
+        for cell in self.ranges[range_index].cells[0..cell_index].iter() {
+            width += cell.width;
+        }
+        width
+    }
+
     #[allow(unused)]
     pub(super) fn last_range_cell_index(&self) -> (usize, usize) {
         let line_ranges_len = self.ranges.len();
@@ -91,7 +107,7 @@ impl Line {
             classes_flag_namespaced(COMPONENT_NAME, class_name_flags)
         };
         let is_focused = text_buffer.is_focused_line(line_index);
-        div(
+        span(
             [
                 key(line_index),
                 class_ns("number__line"),
@@ -100,7 +116,7 @@ impl Line {
             [
                 view_if(
                     text_buffer.options.show_line_numbers,
-                    div(
+                    span(
                         [
                             class_ns("number"),
                             if let Some(gutter_bg) =
@@ -125,7 +141,7 @@ impl Line {
                         [text(line_index + 1)],
                     ),
                 ),
-                div(
+                span(
                     [class_ns("line")],
                     self.ranges.iter().enumerate().map(
                         |(range_index, range)| {
@@ -176,6 +192,8 @@ impl Line {
             let cell = Cell::from_char(ch);
             range.replace_cell(cell_index, cell);
             self.width += range.width;
+        } else {
+            panic!("There should be a range");
         }
     }
 
